[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# FactSet Tick History client library for Python

[![API Version](https://img.shields.io/badge/api-v2.3.0-blue)](https://developer.factset.com/api-catalog/factset-tick-history-api)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.FactSetTickHistory)](https://pypi.org/project/fds.sdk.FactSetTickHistory/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

Tick History provides dynamic access to historical tick data for a specific security for specific dates or date range.

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 2.3.0
- SDK version: 0.32.5
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.FactSetTickHistory==0.32.5
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.FactSetTickHistory==0.32.5
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.FactSetTickHistory
from fds.sdk.FactSetTickHistory.api import level_1_api
from fds.sdk.FactSetTickHistory.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.FactSetTickHistory.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.FactSetTickHistory.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.FactSetTickHistory.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = level_1_api.Level1Api(api_client)
    id = "id_example" # str | id returned by files create endpoint to poll and collect status of the query

    try:
        # Returns the status of the ID
        # example passing only required values which don't have defaults set
        api_response_wrapper = api_instance.get_level1_file_status(id)

        # This endpoint returns a response wrapper that contains different types of responses depending on the query.
        # To access the correct response type, you need to perform one additional step, as shown below.
        if api_response_wrapper.get_status_code() == 201:
            api_response = api_response_wrapper.get_response_201()
        if api_response_wrapper.get_status_code() == 202:
            api_response = api_response_wrapper.get_response_202()

        pprint(api_response)
    except fds.sdk.FactSetTickHistory.ApiException as e:
        print("Exception when calling Level1Api->get_level1_file_status: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Returns the status of the ID
    #     api_response_wrapper, http_status_code, response_headers = api_instance.get_level1_file_status_with_http_info(id)

    #     # This endpoint returns a response wrapper that contains different types of responses depending on the query.
    #     # To access the correct response type, you need to perform one additional step, as shown below.
    #     if api_response_wrapper.get_status_code() == 201:
    #         api_response = api_response_wrapper.get_response_201()
    #     if api_response_wrapper.get_status_code() == 202:
    #         api_response = api_response_wrapper.get_response_202()

    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetTickHistory.ApiException as e:
    #     print("Exception when calling Level1Api->get_level1_file_status: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Returns the status of the ID
    #     async_result = api_instance.get_level1_file_status_async(id)
    #     api_response_wrapper = async_result.get()

    #     # This endpoint returns a response wrapper that contains different types of responses depending on the query.
    #     # To access the correct response type, you need to perform one additional step, as shown below.
    #     if api_response_wrapper.get_status_code() == 201:
    #         api_response = api_response_wrapper.get_response_201()
    #     if api_response_wrapper.get_status_code() == 202:
    #         api_response = api_response_wrapper.get_response_202()

    #     pprint(api_response)
    # except fds.sdk.FactSetTickHistory.ApiException as e:
    #     print("Exception when calling Level1Api->get_level1_file_status: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Returns the status of the ID
    #     async_result = api_instance.get_level1_file_status_with_http_info_async(id)
    #     api_response_wrapper, http_status_code, response_headers = async_result.get()

    #     # This endpoint returns a response wrapper that contains different types of responses depending on the query.
    #     # To access the correct response type, you need to perform one additional step, as shown below.
    #     if api_response_wrapper.get_status_code() == 201:
    #         api_response = api_response_wrapper.get_response_201()
    #     if api_response_wrapper.get_status_code() == 202:
    #         api_response = api_response_wrapper.get_response_202()

    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetTickHistory.ApiException as e:
    #     print("Exception when calling Level1Api->get_level1_file_status: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.FactSetTickHistory

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.FactSetTickHistory.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.FactSetTickHistory

configuration = fds.sdk.FactSetTickHistory.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.FactSetTickHistory

configuration = fds.sdk.FactSetTickHistory.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.FactSetTickHistory

configuration = fds.sdk.FactSetTickHistory.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/tick-history/v2*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*Level1Api* | [**get_level1_file_status**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Level1Api.md#get_level1_file_status) | **GET** /level1/files/status | Returns the status of the ID
*Level1Api* | [**get_minutebars_file_status**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Level1Api.md#get_minutebars_file_status) | **GET** /level1/files/minute-bars/status | Returns the status of the ID
*Level1Api* | [**get_minutebars_files**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Level1Api.md#get_minutebars_files) | **GET** /level1/files/minute-bars/get | Returns the Minute bars data in files requested in the /create endpoint
*Level1Api* | [**get_tick_history_level1_files**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Level1Api.md#get_tick_history_level1_files) | **GET** /level1/files/get | Returns the tick-by-tick data in files requested in the /create endpoint
*Level1Api* | [**request_minutebars_files**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Level1Api.md#request_minutebars_files) | **POST** /level1/files/minute-bars/create | Requests the creation of Minute bars file
*Level1Api* | [**request_tick_history_coverage**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Level1Api.md#request_tick_history_coverage) | **GET** /level1/coverage | Returns the coverage for the requested ticker/isin along with other response fields for Level1 data.
*Level1Api* | [**request_tick_history_level1_files**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Level1Api.md#request_tick_history_level1_files) | **POST** /level1/files/create | Requests the creation of tick-by-tick file


## Documentation For Models

 - [CheckFiles](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/CheckFiles.md)
 - [CompletedResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/CompletedResponse.md)
 - [CoverageResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/CoverageResponse.md)
 - [CoverageTickHistory](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/CoverageTickHistory.md)
 - [DateTimePeriod](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/DateTimePeriod.md)
 - [DateTimePeriodMin](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/DateTimePeriodMin.md)
 - [DateTimeRange](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/DateTimeRange.md)
 - [ErrorExample](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/ErrorExample.md)
 - [ErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/ErrorObject.md)
 - [FactsetExchangeCode](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/FactsetExchangeCode.md)
 - [Fields](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Fields.md)
 - [Files](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Files.md)
 - [FilesOne](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/FilesOne.md)
 - [GetFilesMinuteResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/GetFilesMinuteResponse.md)
 - [GetFilesResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/GetFilesResponse.md)
 - [Granularity](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Granularity.md)
 - [Level1RequestBody](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Level1RequestBody.md)
 - [LeveloneRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/LeveloneRequest.md)
 - [Meta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Meta.md)
 - [MinRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/MinRequest.md)
 - [MinTickData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/MinTickData.md)
 - [MinutesbarRequestBody](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/MinutesbarRequestBody.md)
 - [Pagination](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Pagination.md)
 - [Status](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Status.md)
 - [StatusOne](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/StatusOne.md)
 - [SubmittedResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/SubmittedResponse.md)
 - [TickData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/TickData.md)
 - [Tickers](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Tickers.md)
 - [Type](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetTickHistory/v2/docs/Type.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.FactSetTickHistory.apis and fds.sdk.FactSetTickHistory.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.FactSetTickHistory.api.default_api import DefaultApi`
- `from fds.sdk.FactSetTickHistory.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.FactSetTickHistory
from fds.sdk.FactSetTickHistory.apis import *
from fds.sdk.FactSetTickHistory.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

